@@ -208,7 +208,7 @@ class IpacReader implements RowSequence {
      */
     private static String[] readHeaderLines( LineSequence lseq )
             throws IOException {
-        List lines = new ArrayList();
+        List<String> lines = new ArrayList<String>();
         boolean done = false;
         for ( String line; ! done && ( line = lseq.nextLine() ) != null; ) {
             if ( line.length() > 0 ) {
@@ -221,7 +221,7 @@ class IpacReader implements RowSequence {
                 }
             }
         }
-        return (String[]) lines.toArray( new String[ 0 ] );
+        return lines.toArray( new String[ 0 ] );
     }
 
     /**
@@ -290,8 +290,8 @@ class IpacReader implements RowSequence {
      */
     private static DescribedValue[] readParameters( LineSequence lseq )
             throws IOException {
-        List comments = new ArrayList();
-        List params = new ArrayList();
+        List<String> comments = new ArrayList<String>();
+        List<DescribedValue> params = new ArrayList<DescribedValue>();
         boolean done = false;
         for ( String line; ! done && ( line = lseq.nextLine() ) != null; ) {
             line = line.trim();
@@ -312,8 +312,8 @@ class IpacReader implements RowSequence {
         }
         if ( comments.size() > 0 ) {
             StringBuffer comBuf = new StringBuffer();
-            for ( Iterator it = comments.iterator(); it.hasNext(); ) {
-                comBuf.append( (String) it.next() );
+            for ( Iterator<String> it = comments.iterator(); it.hasNext(); ) {
+                comBuf.append( it.next() );
                 if ( it.hasNext() ) {
                     comBuf.append( '\n' );
                 }
@@ -321,7 +321,7 @@ class IpacReader implements RowSequence {
             params.add( new DescribedValue( IpacTableBuilder.COMMENT_INFO,
                                             comBuf.toString() ) );
         }
-        return (DescribedValue[]) params.toArray( new DescribedValue[ 0 ] );
+        return params.toArray( new DescribedValue[ 0 ] );
     }
 
     /**
@@ -383,7 +383,7 @@ class IpacReader implements RowSequence {
      * @param  value  parameter value
      */
     private static DescribedValue createParameter( String name, Object value ) {
-        Class clazz = value == null ? String.class : value.getClass();
+        Class<?> clazz = value == null ? String.class : value.getClass();
         return new DescribedValue( new DefaultValueInfo( name, clazz ), value );
     }
 
