@@ -11,10 +11,12 @@ import uk.ac.starlink.table.ColumnStarTable;
 import uk.ac.starlink.table.DefaultValueInfo;
 import uk.ac.starlink.table.DescribedValue;
 import uk.ac.starlink.table.JoinStarTable;
+import uk.ac.starlink.table.RowData;
+import uk.ac.starlink.table.RowAccess;
 import uk.ac.starlink.table.RowSequence;
 import uk.ac.starlink.table.StarTable;
-import uk.ac.starlink.table.StreamStarTableWriter;
 import uk.ac.starlink.table.ValueInfo;
+import uk.ac.starlink.table.WrapperRowAccess;
 import uk.ac.starlink.table.WrapperRowSequence;
 import uk.ac.starlink.table.WrapperStarTable;
 import uk.ac.starlink.util.IOUtils;
@@ -28,7 +30,7 @@ import uk.ac.starlink.util.IOUtils;
  * @author   Mark Taylor
  * @since    27 Jul 2006
  */
-public class TstTableWriter extends StreamStarTableWriter {
+public class TstTableWriter extends DocumentedStreamStarTableWriter {
 
     /** Longest field that will get written without truncation. */
     private final static int MAX_CHARS = 10240;
@@ -54,6 +56,10 @@ public class TstTableWriter extends StreamStarTableWriter {
         Y_INFO.setAuxDatum( new DescribedValue( LABEL_INFO, "Y" ) );
     }
 
+    public TstTableWriter() {
+        super( new String[] { "tst" } );
+    }
+
     /**
      * Returns "TST".
      *
@@ -72,17 +78,12 @@ public class TstTableWriter extends StreamStarTableWriter {
         return "text/plain";
     }
 
-    /**
-     * Returns true for <code>location</code> argument which ends in 
-     * ".tst", ".TST" or ".TAB".
-     *
-     * @param   location  the intended destination of the output
-     * @return   whether it looks suitable for this class
-     */
-    public boolean looksLikeFile( String location ) {
-        return location.endsWith( ".tst" ) 
-            || location.endsWith( ".TST" )
-            || location.endsWith( ".TAB" );
+    public boolean docIncludesExample() {
+        return true;
+    }
+
+    public String getXmlDescription() {
+        return readText( "TstTableWriter.xml" );
     }
 
     public void writeStarTable( StarTable st, OutputStream out )
@@ -427,10 +428,8 @@ public class TstTableWriter extends StreamStarTableWriter {
      */
     private static boolean matches( ColumnInfo info1, ColumnInfo info2 ) {
         String labelKey = LABEL_INFO.getName();
-        String label1 = (String) info1.getAuxDatumValueByName( labelKey,
-                                                               String.class );
-        String label2 = (String) info2.getAuxDatumValueByName( labelKey,
-                                                               String.class );
+        String label1 = info1.getAuxDatumValueByName( labelKey, String.class );
+        String label2 = info2.getAuxDatumValueByName( labelKey, String.class );
         return label1 != null && label2 != null && label1.equals( label2 );
     }
 
@@ -522,29 +521,57 @@ public class TstTableWriter extends StreamStarTableWriter {
         }
 
         public RowSequence getRowSequence() throws IOException {
-            return new WrapperRowSequence( super.getRowSequence() ) {
+            RowSequence baseSeq = super.getRowSequence();
+            FactorRow frow = new FactorRow( baseSeq, factors_ );
+            return new WrapperRowSequence( baseSeq ) {
                 public Object getCell( int icol ) throws IOException {
-                    Object val = super.getCell( icol );
-                    return ( factors_[ icol ] == 1.0 || val == null )
-                         ? val
-                         : new Double( ((Number) val).doubleValue()
-                                       * factors_[ icol ] );
+                    return frow.getCell( icol );
                 }
                 public Object[] getRow() throws IOException {
-                    Object[] row = super.getRow();
-                    int ncol = row.length;
-                    for ( int icol = 0; icol < ncol; icol++ ) {
-                        Object val = row[ icol ];
-                        double factor = factors_[ icol ];
-                        if ( factor != 1.0 && val != null ) {
-                            row[ icol ] =
-                                new Double( ((Number) val).doubleValue()
-                                            * factor );
-                        }
-                    }
-                    return row;
+                    return frow.getRow();
+                }
+            };
+        }
+
+        public RowAccess getRowAccess() throws IOException {
+            RowAccess baseAcc = super.getRowAccess();
+            FactorRow frow = new FactorRow( baseAcc, factors_ );
+            return new WrapperRowAccess( baseAcc ) {
+                public Object getCell( int icol ) throws IOException {
+                    return frow.getCell( icol );
+                }
+                public Object[] getRow() throws IOException {
+                    return frow.getRow();
                 }
             };
         }
     }
+
+    private static class FactorRow implements RowData {
+        private final RowData baseRow_;
+        private final double[] factors_;
+        FactorRow( RowData baseRow, double[] factors ) {
+            baseRow_ = baseRow;
+            factors_ = factors;
+        }
+        public Object getCell( int icol ) throws IOException {
+            Object val = baseRow_.getCell( icol );
+            return ( factors_[ icol ] == 1.0 || val == null )
+                ? val
+                : new Double( ((Number) val).doubleValue() * factors_[ icol ] );
+        }
+        public Object[] getRow() throws IOException {
+            Object[] row = baseRow_.getRow();
+            int ncol = row.length;
+            for ( int icol = 0; icol < ncol; icol++ ) {
+                Object val = row[ icol ];
+                double factor = factors_[ icol ];
+                if ( factor != 1.0 && val != null ) {
+                    row[ icol ] =
+                        new Double( ((Number) val).doubleValue() * factor );
+                }
+            }
+            return row;
+        }
+    }
 }
