@@ -46,9 +46,8 @@ public class SystemTableSaveDialog implements TableSaveDialog {
         return true;
     }
 
-    @SuppressWarnings("rawtypes")
     public boolean showSaveDialog( Component parent, final StarTableOutput sto,
-                                   ComboBoxModel formatModel,
+                                   ComboBoxModel<String> formatModel,
                                    StarTable[] tables ) {
 
         /* Construct a FileDialog instance. */
