@@ -26,7 +26,7 @@ import uk.ac.starlink.table.ValueInfo;
  */
 public abstract class Codec {
 
-    private int warnings_;
+    private volatile int warnings_;
     private static Logger logger_ =
         Logger.getLogger( "uk.ac.starlink.table.storage" );
 
@@ -263,7 +263,7 @@ public abstract class Codec {
             return 1;
         }
         public Object decodeObject( ByteStoreAccess in ) throws IOException {
-            return new Byte( in.readByte() );
+            return Byte.valueOf( in.readByte() );
         }
         public int decodeInt( ByteStoreAccess in ) throws IOException {
             return in.readByte();
@@ -288,7 +288,7 @@ public abstract class Codec {
             return 2;
         }
         public Object decodeObject( ByteStoreAccess in ) throws IOException {
-            return new Short( in.readShort() );
+            return Short.valueOf( in.readShort() );
         }
         public int decodeInt( ByteStoreAccess in ) throws IOException {
             return in.readShort();
@@ -313,7 +313,7 @@ public abstract class Codec {
             return 4;
         }
         public Object decodeObject( ByteStoreAccess in ) throws IOException {
-            return new Integer( in.readInt() );
+            return Integer.valueOf( in.readInt() );
         }
         public int decodeInt( ByteStoreAccess in ) throws IOException {
             return in.readInt();
@@ -338,7 +338,7 @@ public abstract class Codec {
             return 8;
         }
         public Object decodeObject( ByteStoreAccess in ) throws IOException {
-            return new Long( in.readLong() );
+            return Long.valueOf( in.readLong() );
         }
         public int decodeInt( ByteStoreAccess in ) throws IOException {
             return (int) in.readLong();
@@ -689,11 +689,9 @@ public abstract class Codec {
     private static class FixedStringCodec extends ObjectCodec {
 
         final int nchar_;
-        final char[] cbuf_;
 
         FixedStringCodec( int nchar ) {
             nchar_ = nchar;
-            cbuf_ = new char[ nchar_ ];
         }
 
         public int encode( Object value, DataOutput out ) throws IOException {
@@ -713,15 +711,16 @@ public abstract class Codec {
 
         public Object decodeObject( ByteStoreAccess in ) throws IOException {
             int lastNonZero = -1;
+            char[] cbuf = new char[ nchar_ ];
             for ( int ic = 0; ic < nchar_; ic++ ) {
                 char c = in.readChar();
                 if ( c != 0 ) {
                     lastNonZero = ic;
                 }
-                cbuf_[ ic ] = c;
+                cbuf[ ic ] = c;
             }
             return lastNonZero < 0 ? null
-                                   : new String( cbuf_, 0, lastNonZero + 1 );
+                                   : new String( cbuf, 0, lastNonZero + 1 );
         }
         public int getItemSize() {
             return nchar_ * 2;
@@ -800,7 +799,7 @@ public abstract class Codec {
      * and deserialize elements of an array.
      */
     private static abstract class Codec1 {
-        private int warnings_;
+        private volatile int warnings_;
 
         /**
          * Serializes an element of an array to a stream.
